@@ -15,11 +15,9 @@ read_h5ad <- function(
   filename,
   backed = NULL
 ) {
-  python_anndata <- reticulate::import("anndata")
-  ad <- AnnData()
-  ad$.__enclos_env__$private$.anndata <- python_anndata$read_h5ad(
+  python_anndata <- reticulate::import("anndata", convert = FALSE)
+  py_to_r(python_anndata$read_h5ad(
     filename = filename,
     backed = backed
-  )
-  ad
+  ))
 }
