@@ -12,11 +12,9 @@
 #' ad <- read_mtx("matrix.mtx")
 #' }
 read_mtx <- function(filename, dtype = "float32") {
-  python_anndata <- reticulate::import("anndata")
-  ad <- AnnData()
-  ad$.__enclos_env__$private$.anndata <- python_anndata$read_mtx(
+  python_anndata <- reticulate::import("anndata", convert = FALSE)
+  py_to_r(python_anndata$read_mtx(
     filename = filename,
     dtype = dtype
-  )
-  ad
+  ))
 }
